package at.jku.ssw.queue.test;

import static org.junit.Assert.assertEquals;

import java.util.NoSuchElementException;

import org.junit.Assert;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.ExpectedException;

import at.jku.ssw.Iterator;
import at.jku.ssw.queue.Queue;

public abstract class QueueTest {

	protected abstract Queue createQueue();

	protected Queue createQueue(Object... objects) {
		Queue queue = createQueue();
		pushValues(queue, objects);
		return queue;
	}

	protected void pushValues(Queue queue, Object... objects) {
		for (Object o : objects) {
			queue.enqueue(o);
		}
	}

	protected void assertContains(Queue queue, Object... objects) {
		Iterator it = queue.iterator();
		int i = 0;
		while (it.hasNext()) {
			Assert.assertEquals(objects[i++], it.next());
		}
		Assert.assertEquals(objects.length, i);
	}

	@Rule
	public ExpectedException thrown = ExpectedException.none();

	@Test
	public void testPush() {
		Queue queue = createQueue(1, 5, 7);
		assertContains(queue, 1, 5, 7);
	}

	@Test
	public void testTake() {
		Queue queue = createQueue(1, 5, 7);
		assertEquals(1, queue.dequeue());
		assertEquals(5, queue.dequeue());
		assertEquals(7, queue.dequeue());
		assertEquals(0, queue.size());

		thrown.expect(NoSuchElementException.class);
		queue.dequeue();
	}

	@Test
	public void testPeek() {
		Queue queue = createQueue(1, 5, 7);
		assertEquals(1, queue.peek());
		assertEquals(1, queue.peek());
		assertEquals(1, queue.peek());
		assertEquals(3, queue.size());
	}

	@Test
	public void testPeekEmpty() {
		Queue queue = createQueue();
		thrown.expect(NoSuchElementException.class);
		queue.peek();
	}

	@Test
	public void testPeekLast() {
		Queue queue = createQueue(1, 5, 7);
		assertEquals(7, queue.peekLast());
		assertEquals(7, queue.peekLast());
		assertEquals(7, queue.peekLast());
		assertEquals(3, queue.size());
	}

	@Test
	public void testPeekLastEmpty() {
		Queue queue = createQueue();
		thrown.expect(NoSuchElementException.class);
		queue.peekLast();
	}

}
